/**
 * Copyright (C) 2018 Silas B. Domingos
 * This source code is licensed under the MIT License as described in the file LICENSE.
 */

/**
 * Autocomplete element interface.
 */
export interface Element extends HTMLDivElement {
  /**
   * Autocomplete name.
   */
  name: string;
  /**
   * Autocomplete value.
   */
  value: any;
  /**
   * Determines whether the autocomplete is empty or not.
   */
  readonly empty: boolean;
  /**
   * Autocomplete search value.
   */
  readonly search: string;
  /**
   * Determines whether the search is remote or not.
   */
  remote: boolean;
  /**
   * Minimum delay to start the search.
   */
  delay: number;
  /**
   * Required state.
   */
  required: boolean;
  /**
   * Read-only state.
   */
  readOnly: boolean;
  /**
   * Disabled state.
   */
  disabled: boolean;
  /**
   * Adds the specified value and its options into the autocompletion results.
   * @param value Unique value.
   * @param option Value text options.
   * @throws Throws an error when the specified value already exists.
   */
  add: (value: string, ...options: string[]) => void;
  /**
   * Clear all search results.
   */
  clear: () => void;
  /**
   * Closes the autocompletion panel.
   */
  close: () => void;
  /**
   * Set the custom validity error message.
   * @param error Custom error message.
   */
  setCustomValidity: (error?: string) => void;
}
