import * as Control from '@singleware/ui-control';
import { Properties } from './properties';
import { Element } from './element';
import { Selection } from './selection';
/**
 * Autocomplete template class.
 */
export declare class Template extends Control.Component<Properties> {
    /**
     * Autocomplete states.
     */
    private states;
    /**
     * Timer Id.
     */
    private timer;
    /**
     * Input slot.
     */
    private inputSlot;
    /**
     * Empty slot.
     */
    private emptySlot;
    /**
     * Loading slot.
     */
    private loadingSlot;
    /**
     * Results slot.
     */
    private resultsSlot;
    /**
     * Autocomplete element.
     */
    private autocomplete;
    /**
     * Autocomplete styles.
     */
    private styles;
    /**
     * Autocomplete skeleton.
     */
    private skeleton;
    /**
     * Autocomplete elements.
     */
    private elements;
    /**
     * Notify input searches.
     * @param input Input element.
     */
    private notifySearch;
    /**
     * Focus event handler.
     */
    private focusHandler;
    /**
     * Blur event handler.
     */
    private blurHandler;
    /**
     * Key up event handler.
     */
    private keyupHandler;
    /**
     * Change event handler.
     */
    private changeHandler;
    /**
     * Bind event handlers to update the custom element.
     */
    private bindHandlers;
    /**
     * Bind exposed properties to the custom element.
     */
    private bindProperties;
    /**
     * Assign all element properties.
     */
    private assignProperties;
    /**
     * Default constructor.
     * @param properties Autocomplete properties.
     * @param children Autocomplete children.
     */
    constructor(properties?: Properties, children?: any[]);
    /**
     * Get autocomplete name.
     */
    /**
    * Set autocomplete name.
    */
    name: string;
    /**
     * Get autocomplete type.
     */
    /**
    * Set autocomplete type.
    */
    type: string;
    /**
     * Get autocomplete value.
     */
    /**
    * Set autocomplete value.
    */
    value: Selection | undefined;
    /**
     * Get empty state.
     */
    readonly empty: any;
    /**
     * Get search value.
     */
    readonly search: any;
    /**
     * Get remote state.
     */
    /**
    * Set remote state.
    */
    remote: boolean;
    /**
     * Get delay state.
     */
    /**
    * Set delay state.
    */
    delay: number;
    /**
     * Get required state.
     */
    /**
    * Set required state.
    */
    required: boolean;
    /**
     * Get read-only state.
     */
    /**
    * Set read-only state.
    */
    readOnly: boolean;
    /**
     * Get disabled state.
     */
    /**
    * Set disabled state.
    */
    disabled: boolean;
    /**
     * Autocomplete element.
     */
    readonly element: Element;
    /**
     * Adds the specified value and its options into the autocompletion results.
     * @param value Unique value.
     * @param option Value text options.
     * @throws Throws an error when the specified value already exists.
     */
    add(value: string, ...options: string[]): void;
    /**
     * Clear all search results.
     */
    clear(): void;
    /**
     * Closes the autocompletion panel.
     */
    close(): void;
    /**
     * Set the custom validity error message.
     * @param error Custom error message.
     */
    setCustomValidity(error?: string): void;
}
